/* ucon backend tunnel:
 * Start up two servers:
 * One that will connect to uCon's tunnel client and another that will
 * act as a telnet server allowing telnet clients to attach to the
 * tunnel and essentially be tied to the serial port that uCon's tunnel
 * client is connected to.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>

int verbose = 0;
int tcsfd = 0;

int
uconwrite(char *buf, int tot)
{
	int i;

	if (verbose >= 2) {
		for(i=0;i<tot;i++)
			fprintf(stderr,"%c",buf[i]);
	}

	send(tcsfd,buf,tot,0);
	return(tot);
}

/* uconTCS():
 * uCon Tunnel-Client-Server
 */
void *
uconTCS(void *arg)
{
	int port, rc;
	int listener;
	socklen_t fromlen;
	struct	sockaddr_in	from;
	struct	sockaddr_in serv_addr;

	port = (int)arg;

	// Open a socket to listen for incoming connections.
	listener = socket (AF_INET, SOCK_STREAM, 0);
	if (listener < 0) {
		err("socket() error");
		return((void *)-1);
	}

	/* Bind a local address so the client can send to us. */
	bzero((char *)&serv_addr,sizeof(struct sockaddr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_addr.sin_port = htons(port);

	rc = bind(listener,(struct sockaddr *)&serv_addr, sizeof(struct sockaddr));

	fprintf(stderr,"UCTS   listening on port %d.\n",port);
	rc = listen (listener, 5);
	if (rc < 0) {
		err("listen() failed");
		return((void *)-1);
	}

	fromlen = sizeof(from);
	if (verbose)
		fprintf(stderr,"UCTS accept...\n");
	tcsfd = accept(listener,(struct sockaddr *)&from,&fromlen);
	if (tcsfd < 0) {
		err("accept() failed");
		return((void *)-1);
	}

	if (verbose)
		fprintf(stderr,"UCTS recv...\n");
	while(1) {
		char c;

		if (recv(tcsfd, &c,1, 0) <= 0) {
			break;
		}

		net_write(&c,1);	
	}

	fprintf(stderr,"Terminating through UCTS\n");
	exit(0);
}

int
main(int argc, char *argv[])
{
	pthread_attr_t attr;
	pthread_t tid = (pthread_t)0;
	char	*password, *welcome;
	int 	i, port, readonly, opt;

	port = 8000;
	readonly = 0;
	password = welcome = 0;

	while((opt=getopt(argc,argv,"P:p:vW:")) != EOF) { 
		switch(opt) {
		case 'W':
			welcome = optarg;
			break;
		case 'P':
			password = optarg;
			break;
		case 'p':
			port = atoi(optarg);
			break;
		case 'v':
			verbose++;
			break;
		default:
			exit(1);
		}
	}

	/* Start up the client-tunnel-server thread:
	 */
	pthread_attr_init(&attr);
	pthread_attr_setstacksize(&attr,0x1000);
	pthread_create(&tid,&attr,uconTCS,(void *)port+1);

	/* Run the telnet server in the main thread:
	 */
	telnetd(password, welcome, port, readonly);
	return 0;
} 

