/* stunc:
 * Very basic STUN client.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <signal.h>
#include <errno.h>

#ifdef BUILD_WITH_VCC
#include <winsock2.h>

#define uint32_t unsigned long
#define sleep(n) Sleep(n*1000)
#define SOCKET_INVALID(s)	(s == INVALID_SOCKET)

char *optarg;
int optind;
int getopt(int argc,char *argv[],char *opts);

WSADATA WsaData;
DWORD	tid = (DWORD)0;
HANDLE	tHandle;

#else

#include <stdint.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define SOCKET_INVALID(s)	(s < 0)

pthread_attr_t attr;
pthread_t tid = (pthread_t)0;
#endif

typedef unsigned char uchar;
typedef unsigned short ushort;

#define STUN_M0		(1 << 0)
#define STUN_M1		(1 << 1)
#define STUN_M2		(1 << 2)
#define STUN_M3		(1 << 3)
#define STUN_C0		(1 << 4)
#define STUN_M4		(1 << 5)
#define STUN_M5		(1 << 6)
#define STUN_M6		(1 << 7)
#define STUN_C1		(1 << 8)
#define STUN_M7		(1 << 9)
#define STUN_M8		(1 << 10)
#define STUN_M9		(1 << 11)
#define STUN_M10	(1 << 12)
#define STUN_M11	(1 << 13)

/* STUN Attribute types:
 *   Comprehension-required range (0x0000-0x7FFF):
 */
#define MAPPED_ADDRESS		0x0001
#define RESPONSE_ADDRESS	0x0002	// now reserved
#define CHANGE_ADDRESS		0x0003	// now reserved
#define SOURCE_ADDRESS		0x0004	// now reserved
#define CHANGED_ADDRESS		0x0005	// now reserved
#define USERNAME			0x0006
#define PASSWORD			0x0007	// now reserved
#define MESSAGE_INTEGRITY	0x0008
#define ERROR_CODE			0x0009
#define UNKNOWN_ATTRIBUTES	0x000a
#define REFLECTED_FROM		0x000b	// now reserved
#define REALM				0x0014
#define NONCE				0x0015
#define XOR_MAPPED_ADDRESS	0x0020

/*  Comprehension-optional range (0x8000-0xFFFF)
 */
#define SOFTWARE			0x8022
#define ALTERNATE_SERVER	0x0823
#define FINGERPRINT			0x0828


#define STUNHDRSIZ	20
#define STUNPORT	3478
#define STUN_MAGICCOOKIE	0x2112A442

struct sockaddr_in targetAddr;

char *errmsg[] = {
	"Usage: natpmp [options] [stunsrvr]",
	" Options:",
	"  -p#    port number to use (default=3478)",
	"  -t     request is for TCP (default is UDP)",
	"  -v     verbose mode",
	"",
	"Note: if no server is specified, default to stunserver.org",
	(char *)0
};

void
usage()
{
	int i = 0;

	while(errmsg[i])
		printf("%s\n",errmsg[i++]);
	exit(1);
}

void
err(char *msg)
{
	if (msg)
		fprintf(stderr,"%s, ",msg);
	perror("");
	exit(-1);
}

char *
stun_attribute_name(ushort attribute)
{
	static char buf[32];

	switch(attribute) {
		case MAPPED_ADDRESS:
			return("MAPPED_ADDRESS");
		case USERNAME:
			return("USERNAME");
		case MESSAGE_INTEGRITY:
			return("MESSAGE_INTEGRITY");
		case ERROR_CODE:
			return("ERROR_CODE");
		case UNKNOWN_ATTRIBUTES:
			return("UNKNOWN_ATTRIBUTES");
		case REALM:
			return("REALM");
		case NONCE:
			return("NONCE");
		case XOR_MAPPED_ADDRESS:
			return("XOR_MAPPED_ADDRESS");
		case SOFTWARE:
			return("SOFTWARE");
		case ALTERNATE_SERVER:
			return("ALTERNATE_SERVER");
		case FINGERPRINT:
			return("FINGERPRINT");
#if 0
		case RESPONSE_ADDRESS:
			return("RESPONSE_ADDRESS (rsvd)");
		case CHANGE_ADDRESS:
			return("CHANGE_ADDRESS (rsvd)");
		case SOURCE_ADDRESS:
			return("SOURCE_ADDRESS (rsvd)");
		case CHANGED_ADDRESS:
			return("CHANGED_ADDRESS (rsvd)");
		case PASSWORD:
			return("PASSWORD (rsvd)");
		case REFLECTED_FROM:
			return("REFLECTED_FROM (rsvd)");
#endif
	}
	sprintf(buf,"0x%04x (unknown)",attribute);
	return(buf);
}

void
stun_process_message(int id, int type, int len, uchar *data)
{
	printf("ATTR%d %s:\n",id,stun_attribute_name(type));
	if (type == MAPPED_ADDRESS)
	{
		ushort port;

		if (data[1] == 0x01)
			printf(" Family: IPv4\n");
		else if (data[1] == 0x02)
			printf(" Family: IPv6\n");
		else
			printf(" Family: unknown (%02x)\n",data[1]);
		port = (ushort)(data[2] << 8) | data[3];
		printf(" Port: %d\n",port);
		if (len == 8)
			printf(" IP: %d.%d.%d.%d\n",data[4],data[5],data[6],data[7]);
		else {
			ushort ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8;
			ip1 = (ushort)(data[4] << 8) | data[5];
			ip2 = (ushort)(data[6] << 8) | data[7];
			ip3 = (ushort)(data[8] << 8) | data[9];
			ip4 = (ushort)(data[10] << 8) | data[11];
			ip5 = (ushort)(data[12] << 8) | data[13];
			ip6 = (ushort)(data[14] << 8) | data[15];
			ip7 = (ushort)(data[16] << 8) | data[17];
			ip8 = (ushort)(data[18] << 8) | data[19];
			printf(" IPv6: %04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x\n",
				ip1, ip2, ip3, ip4, ip5, ip6, ip7, ip8);
		}
	}
	else if (type == SOFTWARE)
	{
		int j;

		putchar(' ');
		for(j=0;j<len;j++)
			putchar(data[j]);
		putchar('\n');
	}
	else
	{
		int j;

		printf(" Value=");
		for(j=0;j<len;j++)
			printf("%02x ",data[j]);
		printf("\n");
	}
}

int
stun_verify_response(int verbose, uchar *rqst, uchar *resp, int len)
{
	int i;

	if (verbose) {
		printf("Resp: %d bytes\n",len);
		printf("HDR:\n ");
		for(i=0;i<STUNHDRSIZ;i++)
			printf("%02x ",resp[i]);
		printf("\nMSG:\n ");
		for(i=STUNHDRSIZ;i<len;i++)
			printf("%02x ",resp[i]);
		printf("\n");
	}

	// Verify that two MSBs are zero:
	if ((resp[0] & 0xc0) != 0x00) {
		fprintf(stderr,"MSB != 00\n");
		return(-1);
	}

	// Verify the magic cookie:
	if ((resp[4] != ((STUN_MAGICCOOKIE & 0xff000000) >> 24)) ||
		(resp[5] != ((STUN_MAGICCOOKIE & 0xff0000) >> 16)) ||
		(resp[6] != ((STUN_MAGICCOOKIE & 0xff00) >> 8)) ||
		(resp[7] != ((STUN_MAGICCOOKIE & 0xff) >> 0))) {
		fprintf(stderr,"Bad Magic-cookie\n");
		return(-1);
	}

	// Verify the transaction ID:
	if (memcmp((char *)&resp[8],(char *)&rqst[8],12) != 0) {
		fprintf(stderr,"Bad transaction id\n");
		return(-1);
	}
	return(0);
}

/* stun_transaction_id():
 * Taken directly out of RFC5389...
 * The transaction ID is a 96-bit identifier, used to uniquely identify
 * STUN transactions.  For request/response transactions, the
 * transaction ID is chosen by the STUN client for the request and
 * echoed by the server in the response.
 */
void
stun_transaction_id(uchar *buf, int classic)
{
	int rval;
	unsigned int t;

	/* Assume buf points to a 12-byte (96-bit) buffer, into which
	 * we assign a transaction id...
	 */
	t = time(0);

	buf[0] = (t & 0x000000ff);
	buf[1] = ((t & 0x0000ff00) >> 8);
	buf[2] = ((t & 0x00ff0000) >> 16);
	buf[3] = ((t & 0xff000000) >> 24);
	
#if BUILD_WITH_VCC
	rval = rand();
#else
	rval = rand_r(&t);
#endif

	buf[4] = (rval & 0x000000ff);
	buf[5] = ((rval & 0x0000ff00) >> 8);
	buf[6] = ((rval & 0x00ff0000) >> 16);
	buf[7] = ((rval & 0xff000000) >> 24);

	t = ~t;

	buf[8] = (t & 0x000000ff);
	buf[9] = ((t & 0x0000ff00) >> 8);
	buf[10] = ((t & 0x00ff0000) >> 16);
	buf[11] = ((t & 0xff000000) >> 24);

	if (classic != 0) {
		rval = ~rval;

		buf[12] = (rval & 0x000000ff);
		buf[13] = ((rval & 0x0000ff00) >> 8);
		buf[14] = ((rval & 0x00ff0000) >> 16);
		buf[15] = ((rval & 0xff000000) >> 24);
	}
}

int
getResponse(int sockfd, uchar *msgbuf, int mbsiz, int waitmsec, int tcp)
{
	int	msglen, rcvlen;
	fd_set	fd_read;
	struct	timeval tval;

	tval.tv_sec = 0;
	tval.tv_usec = waitmsec * 1000;

	FD_ZERO(&fd_read);
	FD_SET(sockfd,&fd_read);

	/* Wait for incoming message: */
	if (select(sockfd+1,&fd_read,0,0,&tval) != 1) {
		return(-1);
	}

	if (tcp) {
		rcvlen = recv(sockfd,msgbuf,mbsiz,0);
	}
	else {
		msglen = sizeof(struct sockaddr);
		rcvlen = recvfrom(sockfd,msgbuf,mbsiz,0,
			(struct sockaddr *)&targetAddr,&msglen);
	}

	if (rcvlen == 0) {
		fprintf(stderr,"Connection closed\n");
	}
	else if (rcvlen < 0) {
		perror("recvfrom");
	}

	return(rcvlen);
}

int
stun_find_servers(char *name)
{
	int i;
	struct	hostent	*hp;

	hp = gethostbyname(name);
	if (hp == NULL) {
		fprintf(stderr,"gethostbyname(%s) failed\n",name);
		exit(1);
	}

	for(i=0;hp->h_aliases[i] != NULL;i++) {
		printf("h_aliases[i]: %s\n", hp->h_aliases[i]);
	}

	if (hp->h_addrtype == AF_INET) {
		struct in_addr addr;

		for(i=0;hp->h_addr_list[i] != NULL;i++) {
			addr.s_addr = *(u_long *) hp->h_addr_list[i++];
			printf("\tIPv4 Address #%d: %s\n", i, inet_ntoa(addr));
		}
	}

#if 0
	for(i=0;hp->h_addr_list[i] != NULL;i++) {
		
#if BUILD_WITH_VCC
		printf("h_addr_list[i]: %s\n",
			inet_ntoa(hp->h_addr_list[i])); 
#else
		printf("h_addr_list[i]: %s\n",
			inet_ntoa( (struct in_addr) *((struct in_addr *) hp->h_addr_list[i]))); 
#endif
    } 
#endif
  
    return 0; 
}
	
int
main(int argc,char **argv)
{
	int useTCP = 0;
	uint32_t inaddr;
	char	*stunsrvr;
	short	stunmsgsize;
	struct	hostent	*hp;
	int		i, opt, pktsiz, len, sfd, verbose, anum, offset;
	uchar	stunrqst[256], stunresp[256];
	ushort	stunmsgtype, stunport;

	verbose = 0;
	stunport = STUNPORT;

	while((opt=getopt(argc,argv,"p:tv")) != EOF) { 
		switch(opt) {
		case 'v':
			verbose = 1;
			break;
		case 'p':
			stunport = atoi(optarg);
			break;
		case 't':
			useTCP = 1;
			break;
		default:
			usage();
		}
	}

#ifdef BUILD_WITH_VCC
	if (WSAStartup (0x0202, &WsaData) == SOCKET_ERROR)
		if (WSAStartup (0x0101, &WsaData) == SOCKET_ERROR)
			err("WSAStartup Failed");
#endif

	if (argc == optind) {
		stunsrvr = "stunserver.org";
	}
	else if (argc == (optind+1)) {
		stunsrvr = argv[optind];
		stun_find_servers(stunsrvr);
		exit(0);
	}
	else {
		usage();
	}

	if (verbose) {
		fprintf(stderr,"Using srvr '%s:%d', in %s mode\n",stunsrvr,stunport,
			useTCP == 0 ? "UDP" : "TCP");
	}

	/* Accept target name as string or internet dotted-decimal address:
	 */
	memset((char *)&targetAddr,0,sizeof(struct sockaddr));
	if ((inaddr = inet_addr(stunsrvr)) != INADDR_NONE) {
		memcpy((char *)&targetAddr.sin_addr,(char *)&inaddr,sizeof(inaddr));
	}
	else {
		hp = gethostbyname(stunsrvr);
		if (hp == NULL) {
			fprintf(stderr,"gethostbyname(%s) failed\n",stunsrvr);
			exit(1);
		}
		memcpy((char *)&targetAddr.sin_addr,hp->h_addr,hp->h_length);
	}

	if (verbose) {
		printf("srvrip: %d.%d.%d.%d\n",
			(targetAddr.sin_addr.s_addr & 0xff) >> 0,
			(targetAddr.sin_addr.s_addr & 0xff00) >> 8,
			(targetAddr.sin_addr.s_addr & 0xff0000) >> 16,
			(targetAddr.sin_addr.s_addr & 0xff000000) >> 24);
	}

	targetAddr.sin_family = AF_INET;
	targetAddr.sin_port = htons(stunport);

	if (useTCP)
		sfd = socket(AF_INET,SOCK_STREAM,0);
	else
		sfd = socket(AF_INET,SOCK_DGRAM,0);

	if (SOCKET_INVALID(sfd))
		err("socket failed");

	if (useTCP) {
		int rc;
		if (verbose)
			fprintf(stderr,"Connecting...");
		rc = connect(sfd,(struct sockaddr *)&targetAddr,
				sizeof(struct sockaddr));
		if (verbose)
			fprintf(stderr,"\n");
		if (rc < 0) {
			perror("connect");
			exit(1);
		}
	}

	/* Start by building the STUN message (starts at offset
	 * 16 in the stunrqst block).  This is done because the
	 * header includes the message size, so we build the header
	 * after we know the message size.
	 */
	stunmsgsize = 0;

	/* Now build the STUN header:
	 */
	stunrqst[0] = 0x00;
	stunrqst[1] = 0x01;
	stunrqst[2] = ((stunmsgsize & 0xff00) >> 8);
	stunrqst[3] = (stunmsgsize & 0xff);
	stunrqst[4] = ((STUN_MAGICCOOKIE & 0xff000000) >> 24);
	stunrqst[5] = ((STUN_MAGICCOOKIE & 0xff0000) >> 16);
	stunrqst[6] = ((STUN_MAGICCOOKIE & 0xff00) >> 8);
	stunrqst[7] = ((STUN_MAGICCOOKIE & 0xff) >> 0);
	
	stun_transaction_id(&stunrqst[8],0);
	
	pktsiz = STUNHDRSIZ + stunmsgsize;
	
	if (verbose)
		fprintf(stderr,"Sending request...");
	if (useTCP) {
		if (send(sfd,stunrqst,pktsiz,0) < 0) {
			close(sfd);
			err("send failed");
		}
		len = getResponse(sfd,stunresp,sizeof(stunresp),2000, useTCP);
	}
	else {
		int waitmsec = 100;

		for(i=0;i<10;i++) {
			if (verbose)
				fprintf(stderr,".");
			if (sendto(sfd,stunrqst,pktsiz,0,
			    (struct sockaddr *)&targetAddr,sizeof(targetAddr)) < 0) {
				close(sfd);
				err("sendto failed");
			}
			if (waitmsec < 2000)
				waitmsec *= 2;
		 	len = getResponse(sfd,stunresp,sizeof(stunresp),waitmsec, useTCP);
			if (len > 0)
				break;
		} 
	}
	if (verbose)
		fprintf(stderr,"\n");

	if (len <= 0) {
		close(sfd);
		fprintf(stderr,"No response from STUN srvr: %d.%d.%d.%d\n",
			(targetAddr.sin_addr.s_addr & 0xff) >> 0,
			(targetAddr.sin_addr.s_addr & 0xff00) >> 8,
			(targetAddr.sin_addr.s_addr & 0xff0000) >> 16,
			(targetAddr.sin_addr.s_addr & 0xff000000) >> 24);
		exit(1);
	}

	/* At this point we've received a response so we follow the steps as
	 * they are outlined in section 7.3 of RFC 5389:
	 */
	if (stun_verify_response(verbose,stunrqst,stunresp,len) < 0) {
		close(sfd);
		err("Message verification failed\n");
	}

	// Read the message type:
	stunmsgtype = (((ushort)(stunresp[0]) << 8) | stunresp[1]);
	printf("msgtype: 0x%04x\n",stunmsgtype);

	// Read each of the received attributes:
	anum = 0;
	offset = STUNHDRSIZ;
	while(offset < len) {
		ushort atype, alen;
		atype = (((ushort)(stunresp[offset]) << 8) | stunresp[offset+1]);
		alen = (((ushort)(stunresp[offset+2]) << 8) | stunresp[offset+3]);
		stun_process_message(anum,atype,alen,stunresp+offset+4);
		offset += alen + 4;
		anum++;
	}
	exit(0);
}

#ifdef BUILD_WITH_VCC
/** \var int getopt_sp
 *	\brief Initialized variable used by getopt.
 */
int	getopt_sp = 1;

/** \var int optopt
 *	\brief Global variable used by getopt.
 */
int	optopt;

/** \var int optind
 *	\brief Global variable used by callers to getopt.  It contains
 *	\brief the index into the argument list that is next.
 */
int	optind = 1;

/** \var char *optarg
 *	\brief Pointer to an argument that has an option.
 */
char	*optarg;


/**	\fn int getopt(int argc,char *argv[],char *opts)
 *	\brief Standard mechanism used to retrieve options (dash-prefixed
 *	\brief letters that provide the program with "optional" 
 *	\brief parameters.
 */
int
getopt(int argc,char *argv[],char *opts)
{
	extern char	*strchr();
	register int c;
	register char *cp;

	if (getopt_sp == 1) {
		char	c0, c1;

		if (optind >= argc)
			return(EOF);

		c0 = argv[optind][0];
		c1 = argv[optind][1];

		if (c0 != '-')
			return(EOF);
		if (c1 == '\0')
			return(EOF);
		if (c0 == '-') {
			if ((isdigit(c1)) && (!strchr(opts,c1)))
				return(EOF);
			if (c1 == '-') {
				optind++;
				return(EOF);
			}
		}
	}
	optopt = c = argv[optind][getopt_sp];
	if (c == ':' || ((cp=strchr(opts, c)) == NULL)) {
		fprintf(stderr,"Illegal '%s' option: %c.\n",argv[0],c);
		if(argv[optind][++(getopt_sp)] == '\0') {
			optind++;
			getopt_sp = 1;
		}
		return('?');
	}
	if (*++cp == ':') {
		if(argv[optind][getopt_sp+1] != '\0')
			optarg = &argv[optind++][getopt_sp+1];
		else if(++(optind) >= argc) {
			fprintf(stderr,
				"Option '%c' of '%s' requires argument.\n",
				c,argv[0]);
			getopt_sp = 1;
			return('?');
		} else
			optarg = argv[optind++];
		getopt_sp = 1;
	} else {
		if(argv[optind][++(getopt_sp)] == '\0') {
			getopt_sp = 1;
			optind++;
		}
		optarg = NULL;
	}
	return(c);
}
#endif
