#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <libgen.h>
#include <signal.h>

// Compile with:
// gcc -fno-builtin -o tsize tsize.c

int oldf;
struct termios oldt, newt;

void
termsetup(void)
{
  tcgetattr(STDIN_FILENO, &oldt);
  newt = oldt;
  newt.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
  fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);
}

void
termcleanup(void)
{
  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
  fcntl(STDIN_FILENO, F_SETFL, oldf);
}

void
alarmfunc(int sig)
{
  termcleanup();
  exit(99);
}

int
main(int argc, char *argv[])
{
        int opt, err, c, i, lines, cols;
        int showrow, showcol;
        char *semi, buf[16], cmd[80];

        showrow = showcol = 0;
        while((opt=getopt(argc,argv,"lrc")) != -1) {
                switch(opt) {
                case 'r':       // rows
                case 'l':       // lines
                        showrow = 1;
                        break;
                case 'c':       // cols
                        showcol = 1;
                        break;
                default:
                        fprintf(stderr,"Usage: %s [-rlc]\n",basename(argv[0]));
                        return(1);
                }
        }
        err = 0;
        termsetup();
        printf("%c[6n",0x1b);
        alarm(2);
        signal(SIGALRM, alarmfunc);
        for(i=0;i<sizeof(buf);i++) {
                do {
                        c = getchar();
                } while(c == EOF);
                if ((i == 0) && (c != 0x1b)) {
                        err=1; break;
                }
                if ((i == 1) && (c != '[')) {
                        err=1; break;
                }
                if (c == 'R')
                        break;
                buf[i] = c;
        }
        termcleanup();
        if (err)
                return(err);
        if (i == sizeof(buf)) {
                return(3);
        }
        semi = (char *)strchr(buf,';');
        if (semi == (char *)0) {
                return(4);
        }
        *semi = 0;
        buf[i] = 0;
        lines = atoi(buf+2);
        cols = atoi(semi+1);

        if ((showrow == 0) && (showcol == 0)) {
                printf("LINES=%d...\n",lines);
                sprintf(cmd,"export LINES=%d",lines);
                system(cmd);
                printf("COLS=%d...\n",cols);
                sprintf(cmd,"export COLS=%d",cols);
                system(cmd);
        }
        else if (showrow)
                printf("%d\n",lines);
        else if (showcol)
                printf("%d\n",cols);

        return(0);

}
